;--------------------------------------------------------------------------
;
;       PETIO.ASM
;       ---------
;
;      *** PETASCII I/O routines for Tapio language C-64 module ***
;       (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Functions :
; (Name)                (Belongs to)            (ECPU registers)
;--------------------------------------------------------------------------
; ** Keyboard(/mouse), joystick and screen functions **
; putc11                System                  None
; print                 User                    r0=offset
; getch                 User                    r0=key pressed
; putch                 User                    r0=mark to output
; jgetchim              User                    r0=key pressed (0 if none)
; input                 User                    r0=Address to input store data
;                                                  (offset) f.e. "inputdata&"
;

; ** Disk functions **
; fopen                 User                    r0=handle r1=drive r2=?
;                                               r3=File name(offset)
; fload                 User                    r0=? r1=drive r2=?
;   \-> Uses ($0330) kernal loader              r3=File name(offset)
; fdrive                User                    r0=Drive number(8-11)
; fclose                User                    r0=handle
; fioinit               User                    None
; fputc                 User                    r0=data
; fgetc                 User                    r0=handle, returns: r0=data
; fcreat                User                    r0=handle r1=drive r2=?
;                                               r3=File name(offset)
;
; Notice: All strings are (PC)ASCIIZ/(64)PETASCIIZ based,
;         so f.e. if you're planning to load some file in memory with fload,
;         then you should first produce an ASCIIZ or PETASCIIZ based string.
;
; (PET)ASCIIZ string example : "Hello world!",0
;
;--------------------------------------------------------------------------
putcbios:   ;BIOS putc
            mov ah,00ah
            mov bh,0
            mov bl,7 ;printing color (?)
            mov cx,1
            int 10h
            ret
;--------------------------------------------------------------------------
;putc:       ;DOS putc
;            mov dl,al
;            mov ah,002h
;            int 21h
;            ret
;--------------------------------------------------------------------------
print:  ;Prints asciiz string to stanard output
        mov di,word [r0]
l000:
        mov al,byte [cs:di]
        cmp al,0
        je  l001
        call putc
        inc di
        jmp l000
l001:   
        ret
;--------------------------------------------------------------------------
getch:  ;Waits for key press and returns key # in r0
        mov ah,008h
        int 21h
        mov ah,0
        mov word [r0],ax
        ret
;--------------------------------------------------------------------------        
putch:  ;DOS putc
        mov dl,al
        mov ah,002h
        int 21h
        mov word [r0],ax
        ret
;--------------------------------------------------------------------------
jgetchim: ; Returns zero, if no key available, other ways it returns key.
        mov ah,1 ; Tests if key is available
        int 016h
;        cmp ax,0
        jz  jgetchim_loop1
        jmp getch ; Read key
;--------------------------------------------------------------------------
jgetchim_loop1:
        mov word [r0],0
        ret
;--------------------------------------------------------------------------



;--------------------------------------------------------------------------
; *** Disk functions ***
;--------------------------------------------------------------------------
f_chan  dw 0
;--------------------------------------------------------------------------
fopen:  mov al,032h ; r0 r1 r2 -- r3=filename
        mov ax,cs
        mov ds,ax
        mov dx,word [r3]
        mov ah,03dh
        int 21h
        mov word [f_chan],ax
        ret
;--------------------------------------------------------------------------
fclose: mov bx,word [f_chan]
        mov ah,03eh
        int 21h
        ret
;--------------------------------------------------------------------------
fioinit: ret
;--------------------------------------------------------------------------
fgetc:  mov bx,word [f_chan]
        mov cx,1
        mov ax,cs
        mov ds,ax
        mov dx,r0
        mov ah,03fh
        int 21h
        ret
;--------------------------------------------------------------------------
fputc:  mov bx,word [f_chan]
        mov cx,1
        mov ax,cs
        mov ds,ax
        mov dx,r0
        mov ah,040h
        int 21h
        ret
;--------------------------------------------------------------------------
fdrive: ret ; ?
;--------------------------------------------------------------------------
fcreat: mov al,03ch ; r0 r1 r2 -- r3=filename
        mov ax,cs
        mov ds,ax
        mov dx,word [r3]
        mov cx,0
        mov ah,03dh
        int 21h
        mov word [f_chan],ax
        ret
;--------------------------------------------------------------------------
;
; JTMOS Real Environment Graphics Library Version 1.00
; 
; (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
;----------------------------------------------------------------------------
jtm_setvmode:           ; al = Video Mode
                        mov byte [jtmvmode],al
                        mov ah,0
                        int 0x10
                        ret
;----------------------------------------------------------------------------
jtm_drawscanline        ; dl = color, si = line
                        push si
                        mov ax,0xa000
                        mov ds,ax
                        imul si,320
                        mov di,si
                        mov cx,320
jtm_drawscanline_l1:
                        mov [ds:di],dl
                        inc di
                        dec cx
                        jnz jtm_drawscanline_l1
                        pop si
                        ret
;----------------------------------------------------------------------------
jtm_paintscreen:        ; dl = color
                        mov si,199
jtm_paintscreen_l1:
                        call jtm_drawscanline
                        dec si
                        cmp si,0xffff
                        jne jtm_paintscreen_l1
                        ret
jtmvmode                db 0,0,0,0
;----------------------------------------------------------------------------
%macro                  jtm_setvgareg 2 ; Register,Data
                        ;
                        mov al,%1
                        mov dx,0x03d4
                        out dx,al
                        inc dx
                        mov al,%2
                        out dx,al
                        ;
%endmacro
;----------------------------------------------------------------------------
%macro                  jtm_getvgareg 1 ; Data
                        ;
                        mov dx,0x03d4
                        in al,dx
                        mov byte [%1],al
                        ;
%endmacro
;----------------------------------------------------------------------------
; Set cursor address (80*y+x)
%macro                  jtm_setcad 2 ; Parameters: x,y
                        mov cx,80
                        imul cx,%2
                        add cx,%1
                        jtm_setvgareg 0x0e,ch
                        jtm_setvgareg 0x0f,cl
%endmacro               
;----------------------------------------------------------------------------
;
; JTMOS Real Environment Ansi Graphics Library Version 1.00
; (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
;----------------------------------------------------------------------------
textcolor               db 0x41
curx                    dw 0
cury                    dw 0
tab                     dw t00,t01,t02,t03,t04,t05
;----------------------------------------------------------------------------
t00 db "JTMOS 0.61 (C)1998 by Jari Tapio Tuominen, all rights reserved.",13,13,0
t01 db "KERNAL located at ",0
t02 db " - ",0
t03 db 13,"JTMOS Command Line Interface enabled.",13,0
t04 db "FREE memory available at ",0
t05 db "Hello Jari, welcome to JTMOS.",13,0
;----------------------------------------------------------------------------
jtmansi_textmode:
                        mov ax,0x03
                        int 0x10
                        mov word [curx],0x00
                        mov word [cury],0x00
                        ret
;----------------------------------------------------------------------------
jtmansi_deleteline:     ; di = line number, ah=color, al=mark
                        push ax
                        imul di,80*2
                        mov ax,0xb800
                        mov es,ax
                        mov cx,80
                        pop ax
jtmansi_deleteline_l1:
                        mov word [es:di],ax
                        add di,2
                        dec cx
                        jnz jtmansi_deleteline_l1
                        ret
;----------------------------------------------------------------------------
jtmansi_scrollscreenup:
                        mov ax,0xb800
                        mov es,ax
                        mov di,0
                        mov cx,80*24
                        mov ax,0
                        mov si,di
                        add si,(80*2)
jtmansi_scrollscreenup_loop1:
                        mov ax,word [es:si]
                        mov word [es:di],ax
                        add si,2
                        add di,2
                        dec cx
                        jnz jtmansi_scrollscreenup_loop1

                        mov di,24
                        mov ah,byte [textcolor]
                        mov al,0
                        call jtmansi_deleteline
                        ret
;----------------------------------------------------------------------------
jtmansi_enter:          ; Acts like enter is pressed or asc code 13 sent.
                        mov word [curx],0
                        cmp word [cury],24
                        jne jtmansi_enter_n24
                        mov di,24
                        call jtmansi_scrollscreenup
                        ret
jtmansi_enter_n24
                        inc word [cury]
                        ret
;----------------------------------------------------------------------------
jtmansi_gotoxy:         ; ax,bx = x,y
                        mov word [curx],ax
                        mov word [cury],bx
                        jtm_setcad ax,bx
                        ret
;----------------------------------------------------------------------------
jtmansi_textcolor:
                        ; al = color
                        mov byte [textcolor],al
                        ret
;----------------------------------------------------------------------------
jtmansi_cursorup:
                        cmp word [cury],0
                        je  jtmansi_cursorup_zero
                        dec word [cury]
jtmansi_cursorup_zero:
                        ret
;----------------------------------------------------------------------------
jtmansi_decreasecursorpos:
                        dec word [curx]
                        cmp word [curx],-1
                        jne nminus
                        mov word [curx],0
                        call jtmansi_cursorup
nminus:
                        ret
;----------------------------------------------------------------------------
jtmansi_increasecursorpos:
                        inc word [curx]
                        cmp word [curx],80
                        jne n80
                        call jtmansi_enter
n80:
                        ret
;----------------------------------------------------------------------------
%macro                  stextcolor 1
                        mov al,%1
                        call jtmansi_textcolor
%endmacro
;----------------------------------------------------------------------------
; Set new textcolor (Notice: higher 4 bits are background)
txtcolor:       
                        mov al,byte [r0]
                        stextcolor al
                        ret
;----------------------------------------------------------------------------
%macro                  gotoxy 2
                        ;
                        mov ax,%1
                        mov bx,%2
                        call jtmansi_gotoxy
                        ;
%endmacro
;----------------------------------------------------------------------------
%macro                  putchar 1
                        ;
                        mov al,%1
                        call putc
                        ;

%endmacro
;----------------------------------------------------------------------------
jtmansi_putc1:          ; ah = color, al = mark
                        cmp al,13
                        jne jtmansi_putc1_nenter
                        jmp jtmansi_enter
jtmansi_putc1_nenter
                        cmp al,8
                        jne jtmansi_putc1_nodel                       
                        call jtmansi_decreasecursorpos
                        mov al,0x20
                        jmp jtmansi_putatxy
jtmansi_putc1_nodel:
                        call jtmansi_putatxy
                        call jtmansi_increasecursorpos
                        ret
;----------------------------------------------------------------------------
jtmansi_putatxy:
                        push ax
                        mov ax,0xb800
                        mov es,ax

                        mov di,word [cury]
                        imul di,(0x28*2)
                        add di,word [curx]
                        shl di,1

                        pop ax
                        mov word [es:di],ax
                        ;
                        ret
;----------------------------------------------------------------------------
jtmansi_clearscreen:    ; AH = (High)Background (Low)Forgeground color
                        mov cx,0xb800
                        mov es,cx
                        mov di,0
                        mov cx,80*25
jtmansi_clearscreen_loop1:
                        mov word [es:di],ax
                        add di,2
                        dec cx
                        jnz jtmansi_clearscreen_loop1
                        ret
;----------------------------------------------------------------------------
putc:                   ; al = mark, color defined in textcolor
                        mov ah,byte [textcolor]
                        call jtmansi_putc1
                        jtm_setcad word [curx],word [cury]
                        ret
;----------------------------------------------------------------------------
jtmansi_printo:         ; SI => 16bit offset to text (CS segment used)

                        mov word [jtmansi_printo_addy],si
jtmansi_printo_loop1:
                        mov si,word [jtmansi_printo_addy]
                        mov al,[cs:si]
                        ;
                        cmp al,0
                        je  jtmansi_printo_zer
                        call putc
                        ;
                        inc word [jtmansi_printo_addy]
                        jmp jtmansi_printo_loop1
jtmansi_printo_zer:
                        ret

jtmansi_printo_addy     dw 0
;----------------------------------------------------------------------------
messageman:             ; si=message number to print
                        shl si,1
                        add si,tab

                        mov ax,word [cs:si]
                        mov si,ax

                        call jtmansi_printo
                        ret
;----------------------------------------------------------------------------
jtmansi_input_string    db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
jtmansi_input_pointer   dw 0
jtmansi_input_pointer2  dw 0
jtmansi_input:          ; Input routine
                        ;
                        mov ax,jtmansi_input_string
                        mov word [jtmansi_input_pointer],ax
                        mov ax,0
                        mov word [jtmansi_input_pointer2],ax
jtmansi_input_l1:                
                        call jtm_waitkey

                        ; ESC ? ENTER ?
                        cmp al,27
                        je  jtmansi_input_quit
                        cmp al,13
                        je  jtmansi_input_quit

                        ; DEL ?
                        cmp al,8
                        jne jtmansi_input_ndel
                        mov cx,word [jtmansi_input_pointer2]
                        cmp cx,0
                        je  jtmansi_input_l1
                        dec word [jtmansi_input_pointer]
                        dec word [jtmansi_input_pointer2]
                        call putc
                        jmp jtmansi_input_l1
jtmansi_input_ndel:
                        mov di,word [jtmansi_input_pointer]
                        mov byte [cs:di],al
                        call putc

                        inc word [jtmansi_input_pointer]
                        inc word [jtmansi_input_pointer2]

                        jmp jtmansi_input_l1
jtmansi_input_quit:
                        mov di,word [jtmansi_input_pointer]
                        mov byte [cs:di],0
                        ret
;----------------------------------------------------------------------------
jtmansi_writehex32:
                ; Prints value in register EAX.
                ;
                push eax
                shr eax,16
                call jtmansi_writehex16
                pop eax
                call jtmansi_writehex16
                ;
                ret
;----------------------------------------------------------------------------
;
writehex16:
                mov ax,word [r0]
                jmp jtmansi_writehex16
;----------------------------------------------------------------------------
jtmansi_writehex16_tab  db '0123456789abcdef'
jtmansi_writehex16_value16 dw 0
jtmansi_writehex16:     ; Writes 16bit hex value on screen, ax=value (16bit)
                ;
                mov word [jtmansi_writehex16_value16],ax
                ;
                mov si,word [jtmansi_writehex16_value16]
                shr si,(3*4)
                and si,15
                add si,jtmansi_writehex16_tab
                mov al,byte [cs:si]
                call putc
                ;
                mov si,word [jtmansi_writehex16_value16]
                shr si,(2*4)
                and si,15
                add si,jtmansi_writehex16_tab
                mov al,byte [cs:si]
                call putc
                ;
                mov si,word [jtmansi_writehex16_value16]
                shr si,(1*4)
                and si,15
                add si,jtmansi_writehex16_tab
                mov al,byte [cs:si]
                call putc
                ;
                mov si,word [jtmansi_writehex16_value16]
                shr si,(0*4)
                and si,15
                add si,jtmansi_writehex16_tab
                mov al,byte [cs:si]
                call putc
                ret
;----------------------------------------------------------------------------
jtmansi_writehexsegment_p1 dw 0
jtmansi_writehexsegment_p2 dw 0
jtmansi_writehexsegment: ; AX:BX => Segment:Offset
                ;
                mov word [jtmansi_writehexsegment_p1],ax
                mov word [jtmansi_writehexsegment_p2],bx
                ;
                mov ax,word [jtmansi_writehexsegment_p1]
                call jtmansi_writehex16
                mov al,':'
                call putc
                mov ax,word [jtmansi_writehexsegment_p2]
                call jtmansi_writehex16
                ;
                ret
;----------------------------------------------------------------------------
jtmansi_memorymap: ; Displays current memory status


                ; DISPLAY KERNAL LOCATION !
                mov al,0x0e
                call jtmansi_textcolor
                ;
                mov si,1
                call messageman
                ;
                mov ax,cs
                mov bx,___sof___
                call jtmansi_writehexsegment
                ;
                mov si,2
                call messageman
                ;
                mov ax,cs
                mov bx,kernaleoa
                call jtmansi_writehexsegment
                mov al,13
                call putc

                ; DISPLAY FREE MEMORY MAP
                mov al,0x0e
                call jtmansi_textcolor
                ;
                mov si,4
                call messageman
                ;
                mov ax,cs
                mov bx,kernaleoa
                call jtmansi_writehexsegment
                ;
                mov si,2
                call messageman
                ;
                mov ax,0xa000
                mov bx,0x0000
                call jtmansi_writehexsegment
                mov al,13
                call putc
                ret
;----------------------------------------------------------------------------
jtm_waitkey:
                mov ah,0x00
                int 0x16
                ret
;----------------------------------------------------------------------------
input:          ; Input routine
                        ;
                        mov ax,jtmansi_input_string
                        mov word [jtmansi_input_pointer],ax
                        mov ax,0
                        mov word [jtmansi_input_pointer2],ax
input_l1:                
                        call jtm_waitkey

                        ; ESC ? ENTER ?
                        cmp al,27
                        je  input_quit
                        cmp al,13
                        je  input_quit

                        ; DEL ?
                        cmp al,8
                        jne input_ndel
                        mov cx,word [jtmansi_input_pointer2]
                        cmp cx,0
                        je  input_l1
                        dec word [jtmansi_input_pointer]
                        dec word [jtmansi_input_pointer2]
                        call putc
                        jmp input_l1
input_ndel:
                        mov di,word [jtmansi_input_pointer]
                        mov byte [cs:di],al
                        call putc

                        inc word [jtmansi_input_pointer]
                        inc word [jtmansi_input_pointer2]

                        jmp input_l1
input_quit:
                        mov di,word [jtmansi_input_pointer]
                        mov byte [cs:di],0
                        ret
