;--------------------------------------------------------------------------
;
;       PETIO.ASM
;       ---------
;
;      *** PETASCII I/O routines for Tapio language C-64 module ***
;       (C)1998 by Jari Tapio Tuominen, all rights reserved.
;
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
; Functions :
; (Name)                (Belongs to)            (ECPU registers)
;--------------------------------------------------------------------------
; ** Keyboard(/mouse), joystick and screen functions **
; putc11                System                  None
; print                 User                    r0=offset
; getch                 User                    r0=key pressed
; putch                 User                    r0=mark to output
; jgetchim              User                    r0=key pressed (0 if none)
; input                 User                    r0=Address to input store data
;                                                  (offset) f.e. "inputdata&"
;

; ** Disk functions **
; fopen                 User                    r0=handle r1=drive r2=?
;                                               r3=File name(offset)
; fload                 User                    r0=? r1=drive r2=?
;   \-> Uses ($0330) kernal loader              r3=File name(offset)
; fdrive                User                    r0=Drive number(8-11)
; fclose                User                    r0=handle
; fioinit               User                    None
; fputc                 User                    r0=data
; fgetc                 User                    r0=handle, returns: r0=data
; fcreat                User                    r0=handle r1=drive r2=?
;                                               r3=File name(offset)
;
; Notice: All strings are (PC)ASCIIZ/(64)PETASCIIZ based,
;         so f.e. if you're planning to load some file in memory with fload,
;         then you should first produce an ASCIIZ or PETASCIIZ based string.
;
; (PET)ASCIIZ string example : "Hello world!",0
;
;--------------------------------------------------------------------------
putcbios:   ;BIOS putc
            mov ah,00ah
            mov bh,0
            mov bl,7 ;printing color (?)
            mov cx,1
            int 10h
            ret
;--------------------------------------------------------------------------
putc:       ;DOS putc
            mov dl,al
            mov ah,002h
            int 21h
            ret
;--------------------------------------------------------------------------
print:  ;Prints asciiz string to stanard output
        mov di,r0
l000:
        mov al,byte ptr [cs:di]
        cmp al,0
        je  l001
        call putc
        inc di
        jmp l000
l001:   
        ret
;--------------------------------------------------------------------------
getch:  ;Waits for key press and returns key # in r0
        mov ah,008h
        int 21h
        mov ah,0
        mov word ptr r0,ax
        ret
;--------------------------------------------------------------------------        
putch:  ;DOS putc
        mov dl,al
        mov ah,002h
        int 21h
        mov word ptr r0,ax
        ret
;--------------------------------------------------------------------------
jgetchim: ; Returns zero, if no key available, other ways it returns key.
        mov ah,1 ; Tests if key is available
        int 016h
;        cmp ax,0
        jz  jgetchim_loop1
        jmp getch ; Read key
;--------------------------------------------------------------------------
jgetchim_loop1:
        mov r0,0
        ret
;--------------------------------------------------------------------------



;--------------------------------------------------------------------------
; *** Disk functions ***
;--------------------------------------------------------------------------
f_chan  dw 0
;--------------------------------------------------------------------------
fopen:  mov al,032h ; r0 r1 r2 -- r3=filename
        mov ax,cs
        mov ds,ax
        mov dx,r3
        mov ah,03dh
        int 21h
        mov f_chan,ax
        ret
;--------------------------------------------------------------------------
fclose: mov bx,f_chan
        mov ah,03eh
        int 21h
        ret
;--------------------------------------------------------------------------
fioinit: ret
;--------------------------------------------------------------------------
fgetc:  mov bx,f_chan
        mov cx,1
        mov ax,cs
        mov ds,ax
        mov dx,offset r0
        mov ah,03fh
        int 21h
        ret
;--------------------------------------------------------------------------
fputc:  mov bx,f_chan
        mov cx,1
        mov ax,cs
        mov ds,ax
        mov dx,offset r0
        mov ah,040h
        int 21h
        ret
;--------------------------------------------------------------------------
fdrive: ret ; ?
;--------------------------------------------------------------------------
fcreat: mov al,03ch ; r0 r1 r2 -- r3=filename
        mov ax,cs
        mov ds,ax
        mov dx,r3
        mov cx,0
        mov ah,03dh
        int 21h
        mov f_chan,ax
        ret
;--------------------------------------------------------------------------

;--------------------------------------------------------------------------
jtmansi_input_string    db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef"
                        db "0123456789abcdef0123456789abcdef" ; Requires alot space!
                        db "0123456789abcdef0123456789abcdef"
jtmansi_input_pointer   dw 0
jtmansi_input_pointer2  dw 0
                        ;
input:                  mov ax,word ptr jtmansi_input_string
                        mov word ptr jtmansi_input_pointer,ax
                        mov ax,0
                        mov word ptr jtmansi_input_pointer2,ax
jtmansi_input_l1:       call getch
                        ; ESC ? ENTER ?
                        mov ax,word ptr r0
                        cmp al,27
                        je  jtmansi_input_quit
                        cmp al,13
                        je  jtmansi_input_quit

                        ; DEL ?
                        cmp al,8
                        jne jtmansi_input_ndel
                        mov cx,word ptr jtmansi_input_pointer2
                        cmp cx,0
                        je  jtmansi_input_l1
                        dec word ptr jtmansi_input_pointer
                        dec word ptr jtmansi_input_pointer2
                        call putc
                        jmp jtmansi_input_l1
jtmansi_input_ndel:
                        mov di,word ptr jtmansi_input_pointer
                        mov byte ptr [cs:di],al
                        call putc

                        inc word ptr jtmansi_input_pointer
                        inc word ptr jtmansi_input_pointer2

                        jmp jtmansi_input_l1
jtmansi_input_quit:
                        mov di,word ptr jtmansi_input_pointer
                        mov byte ptr [cs:di],0
                        ret
;--------------------------------------------------------------------------
